// joi/10/juno/GUILoginConsole.java                         
//                                                            
//                                                            
// Copyright 2003 Bill Campbell and Ethan Bolker                         
                                                            
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

/**
 *  The graphical user interface to Juno.
 */

public class GUILoginConsole extends JFrame
    implements OutputInterface
{
    private static final int FIELDWIDTH = 30;
    private static final int FIELDHEIGHT = 5;

    private final Juno junoSystem;
    private WindowCloser closeMe; // to shut down Juno

    private String title;  // title for the windows

    // The interpreter interprets one-line commands.
    private InterpreterInterface interpreter; 
    private boolean echoInput;

    // All output goes to messages.
    private JTextArea messages;

    /**
     *  Construct a GUI console for Juno.
     *
     *  @param title the title for this window.
     *  @param junoSystem the Juno system for which this is a GUI
     *  @param interpreter the object to which to send user input.
     *  @param echoInput true when input echoes to this console.
     */

    public GUILoginConsole( String title, Juno junoSystem,
			    InterpreterInterface interpreter,
			    boolean echoInput) 
    { 
	super( title );
	this.title       = title;
	this.junoSystem  = junoSystem;
	this.interpreter = interpreter;
	this.echoInput   = echoInput;
	this.closeMe = new WindowCloser( junoSystem );

	// Set up the look and feel;
	// Everthing is placed on a panel (using BorderLayout)

	JPanel panel = new JPanel();
	panel.setLayout( new BorderLayout() );

	// First a tabbed pane, with two tabs:
	//   one for login, one for registration

	JTabbedPane tabs = new JTabbedPane();
	tabs.addTab( "Login" , null, 
		     new LoginPane( interpreter, echoInput, closeMe ) );
	tabs.addTab( "Register", null,
		     new RegisterPane( interpreter, echoInput ) );
	tabs.setSelectedIndex( 0 );  // Login selected by default
	panel.add( tabs, BorderLayout.NORTH );

	// and the output messages area.
	panel.add( new JLabel( "Messages:" ), BorderLayout.CENTER );
	messages = new JTextArea( FIELDHEIGHT, FIELDWIDTH );
	panel.add( messages, BorderLayout.SOUTH );

	// Add the panel to this JFrame
	this.getContentPane().add( panel );

	// Closing this window
	this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	this.addWindowListener( closeMe );

	// Size and display this JFrame
	pack();
	show();
    }

    // Implementing the OutputInterface. Everything goes to the
    // single message area.

    /**
     * Write a String followed by a newline
     * to message area.
     *
     * @param str - the string to write
     */

    public void println(String str ) 
    {
        messages.append( str + "\n" );
    }
    
    /**
     * Write a String followed by a newline
     * to message area.
     *
     * @param str - the String to write
     */

    public void errPrintln(String str ) 
    {
        println( str );
    }

    /**
     * Query what kind of console this is.
     *
     * @return true if and only if echoing input.
     */

    public boolean isEchoInput() 
    {
	return echoInput;
    }

    /**
     * Query what kind of console this is.
     *
     * @return true if and only if GUI
     */

    public boolean isGUI() 
    {
	return true;
    }

    /**
     * Query what kind of console this is.
     *
     * @return true if and only if remote
     */

    public boolean isRemote() 
    {
	return false;
    }
	
    // The Login pane is specified in a private inner class,
    // visible only here.

    private class  LoginPane extends JPanel
    {
	// The login pane has two text fields and two buttons.
	private JTextField nameField; 
	private JTextField passwordField;
	
	private JButton ok;
	private JButton exit;

	private WindowCloser closeMe; // to shut down Juno
	// Construct the login pane and set up its listeners.

	public LoginPane( InterpreterInterface interpreter,
			  boolean echoInput, WindowCloser closeMe ) 
	{ 
	    super();
	    this.closeMe = closeMe;

	    // Set up the look and feel.
	    // Everything will go into a vertical Box, a container
	    // whose contents are laid out using BoxLayout

	    Box box = Box.createVerticalBox();

	    // First a panel, containing the two text fields

	    JPanel p  = new JPanel();
	    p.setLayout( new GridLayout( 4 , 1 ) );

	    p.add( new JLabel( "Login:" ) );
	    nameField = new JTextField( FIELDWIDTH );
	    p.add( nameField );

	    p.add( new JLabel( "Password:" ) );
	    passwordField = new JPasswordField( FIELDWIDTH );
	    p.add( passwordField );
	    box.add( p );
	    box.add( Box.createVerticalStrut( 15 ) );

	    // Then a horizontal Box containing the two buttons

	    Box row = Box.createHorizontalBox();
	    row.add( Box.createGlue() );
	    
	    ok = new JButton( "OK" );
	    row.add( ok );
	    row.add( Box.createGlue() );

	    exit = new JButton( "Exit" );
	    row.add( exit );
	    row.add( Box.createGlue() );
	    box.add( row );
	    box.add( Box.createVerticalStrut( 15 ) );

	    this.setLayout( new BorderLayout() );
	    this.add( box, BorderLayout.CENTER );

	    // Set up the listeners (the semantics)

	    ok.addActionListener( new LoginProcessor() );
	    exit.addActionListener( closeMe ); // shuts down Juno
	}

	// An inner inner class for the semantics 
	// when the user clicks OK.

	private class LoginProcessor implements ActionListener 
	{
	    public void actionPerformed(ActionEvent e) 
	    {
		String str = nameField.getText() +  " " +
		    passwordField.getText();
		nameField.setText("");
		passwordField.setText("");
		messages.setText(str+'\n');  // for debugging
		interpreter.interpret( str );
	    }
	}
    }

    // The Register pane is specified in a private inner class,
    // visible only here.

    private class  RegisterPane extends JPanel
    {
	// The register pane has four textfields and two buttons.
	private JTextField chosenName;
	private JTextField fullName;
	private JTextField password1;
	private JTextField password2;
	
	private JButton register;
	private JButton clear;

	public RegisterPane( InterpreterInterface interpreter,
			     boolean echoInput) 
	{ 
	    super();

	    // Define the look and feel
	    // Everything goes into a vertical Box
	    Box box = Box.createVerticalBox();

	    // First a panel containing the text fields

	    JPanel p  = new JPanel();
	    p.setLayout( new GridLayout( 0 , 1 ) );
	    
	    p.add( new JLabel( "Choose login name:" ) );
	    chosenName = new JTextField( FIELDWIDTH );
	    p.add( chosenName );
	    
	    p.add( new JLabel( "Give full name:" ) );
	    fullName = new JTextField( FIELDWIDTH );
	    p.add( fullName );
	    
	    p.add( new JLabel( "Choose password:" ) );
	    password1  = new JPasswordField( FIELDWIDTH );
	    p.add( password1 );
	    
	    p.add( new JLabel( "Retype password:" ) );
	    password2  = new JPasswordField( FIELDWIDTH );
	    p.add( password2 );

	    box.add( p );
	    box.add( Box.createVerticalStrut( 15 ) );

	    // Then a horizontal Box containing the buttons
	    
	    Box row = Box.createHorizontalBox();
	    row.add( Box.createGlue() );
	    register = new JButton( "Register" );
	    row.add( register );
	    row.add( Box.createGlue() );
	    clear = new JButton( "Clear" );
	    row.add( clear );
	    row.add( Box.createGlue() );
	    box.add( row );
	    box.add( Box.createVerticalStrut( 15 ) );
	    
	    this.setLayout( new BorderLayout() );
	    this.add( box, BorderLayout.CENTER );

	    // Set up the listeners (the semantics)

	    register.addActionListener( new Registration() );
	    clear.addActionListener( new Cleanser() );
	}

        // An inner inner class for the semantics when the user 
	// clicks Register.

        private class Registration implements ActionListener 
	{
            public void actionPerformed(ActionEvent e) 
	    {
		if ( password1.getText().trim().equals(
 	             password2.getText().trim() ) ) {
		    String str = "register " +
			chosenName.getText() +  " " +
			password1.getText() + " " +
			fullName.getText() ;
		    chosenName.setText("");
		    fullName.setText("");
		    messages.setText(str+'\n');  // for debugging
		    interpreter.interpret(str);
		}
		else {
		    messages.setText( 
		        "Sorry, passwords don't match.\n" );
		}
		password1.setText("");
		password2.setText("");
            }
        }

        // An inner inner class for the semantics when the user 
	// clicks Clear.

        private class Cleanser implements ActionListener {
            public void actionPerformed(ActionEvent e) {
		chosenName.setText("");
		fullName.setText("");
		password1.setText("");
                password2.setText("");
            }
        }
    }

    //  A WindowCloser instance handles close events generated 
    //  by the underlying window system with its windowClosing
    //  method, and close events from buttons or other user
    //  components with its actionPerformed method.
    //
    //  The action is to shut down Juno.

    private static class WindowCloser extends WindowAdapter
        implements ActionListener  
    {
	Juno system;

        public WindowCloser( Juno system ) 
	{
	    this.system = system;
        }

        public void windowClosing (WindowEvent e) 
	{
            this.actionPerformed( null ); 
        }

        public void actionPerformed(ActionEvent e) 
	{
	    if (system != null) {
		system.shutDown();
	    }
	    System.exit(0);
        }
    }

    /**
     * main() in GUILoginConsole class for 
     * unit testing during development.
     */  

    public static void main( String[] args ) 
    {
	new GUILoginConsole( "GUItest", null, null, true ).show();
    }
}

