// joi/10/juno/RemoteConsole.java                         
//                                                            
//                                                            
// Copyright 2003 Bill Campbell and Ethan Bolker                         
                                                            
import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;

/**
 * A remote console listens on a port for a remote login to
 * a running Juno system server. 
 *
 * @version 10
 */

public class RemoteConsole extends Thread 
    implements OutputInterface, InputInterface 
{
    // default just logs connection start and end
    // change to true to log all i/o
    private static boolean logall = false;

    private Juno system;
    private boolean echo;
    private InterpreterInterface interpreter;

    private Socket clientSocket;  
    private BufferedReader in;
    private PrintWriter out;
    private int sessionCount = 0;

    private PrintWriter junoLog;

    /**
     * Construct a remote console to listen for users trying
     * to connect to Juno. 
     *
     * Called from Juno main.
     *
     * @param system the Juno system setting up this console.
     * @param echo whether or not input should be echoed.
     * @param port the port on which to listen for requests.
     */

    public RemoteConsole( Juno system, boolean echo, int port ) 
    {
	this.echo = echo;
	Date now  = new Date(); 
	junoLog   = openlog(now);
	log("*** Juno server started " + now + "\n");
	try {
	    ServerSocket ss = new ServerSocket(port);
	    while (true) {
		clientSocket = ss.accept(); 
		new RemoteConsole( system, echo, clientSocket,
				   junoLog, ++sessionCount).start();
	    }
	}
	catch (IOException e) {
	    System.out.println("Remote login not supported");
	    System.exit(0);
	}
	finally {
	    system.shutDown();
	}
    }

    /** 
     * Construct a remote console for a single remote user.
     *
     * @param system the Juno system to which the user is connecting.
     * @param echo whether or not input should be echoed.
     * @param clientSocket the socket for the user's connection
     * @param junoLog track all user i/o
     * @param sessionCount this session's number 
     */

    public RemoteConsole( Juno system, boolean echo, Socket clientSocket, 
			  PrintWriter junoLog, int sessionCount ) 
    {
	this.system = system;
	this.echo = echo;
	this.clientSocket = clientSocket;
	this.junoLog = junoLog;
	this.sessionCount = sessionCount;
    }

    /**
     * Action when the thread for this session starts.
     */

    public void run() 
    {
	log("*** " + sessionCount + ", " +
	    clientSocket.getInetAddress()  + ", "  +
	    new Date());
	try {
	    setUpConnection();
	    String s = this.readLine
		("Please sign the guest book (name, email): ");
	    this.println("Thanks, " + s);	
	    if (!logall) {
		log("guest book: " + s);
	    }
	    new LoginInterpreter( system, this ).CLILogin();
	    clientSocket.close();
	}
	catch (IOException e) {
	    log("*** Error " + e);
	}
	log("*** end session " + sessionCount); 
    }

    /**
     * Create the readers and writers for the socket 
     * for this session.
     */

    private void setUpConnection() 
	 throws IOException 
    {
	in = new BufferedReader(
	        new InputStreamReader(clientSocket.getInputStream()));
	out = new PrintWriter(
	        new OutputStreamWriter(clientSocket.getOutputStream()));
    }
    
    // implement the InputInterface

   /**
    * Read a line (terminated by a newline) from console socket.
    *
    * Log the input line before returning it if required.  
    *
    * @param promptString output string to prompt for input
    * @return the string (without the newline character)
    */
    
    public String readLine( String promptString ) 
    {
	String s = "";
	this.print(promptString);
	out.flush();
	try {
	    s = in.readLine();
	    if (logall) { 
		log("> " + s);
	    }
	    if (echo) {
		out.println(s);
	    }
	}
	catch (IOException e) {
	    String msg = "IO error reading from remote console";
	    System.out.println(msg);
	    out.println(msg);
	}
	return s;
    }

    /**
     * Write a String to console socket.
     *
     * Log the output if required.  
     *
     * @param str - the string to write
     */

    public void print( String str ) 
    {
	out.print( str );
	out.flush();
	if (logall) {
	    log("< " + str + "\\");
	}
    }

    // implement the OutputInterface

    /**
     * Write a String followed by a newline
     * to console socket.
     *
     * Log the output if required.  
     *
     * @param str - the string to write
     */

    public void println( String str ) 
    {
	out.println( str + '\r' );
	out.flush();
	if (logall) {
	    log("< " + str);
	}
    }

    /**
     * Write a String followed by a newline
     * to console error output location. That's
     * just the socket.
     *
     * @param str - the String to write
     */

    public void errPrintln(String str ) 
    {
	println( str );
    }

    /**
     * Query what kind of console this is.
     *
     * @return false since it's not a GUI.
     */

    public boolean isGUI() 
    {
	return false;
    }

    /**
     * Query what kind of console this is.
     *
     * @return true since it is remote.
     */

    public boolean isRemote() 
    {
	return true;
    }

    /**
     * Query what kind of console this is.
     *
     * @return true if and only if echoing input.
     */

    public boolean isEchoInput() 
    {
	return echo;
    }

    /**
     * Log a String.
     *
     * @param str the String to log.
     */

    private void log(String str) 
    {
	junoLog.println(sessionCount + ": " + str);
	junoLog.flush();
    }

    /**
     * Open a log for this console.
     *
     * @param now the current Date.
     */

    private PrintWriter openlog(Date now) 
    {
	PrintWriter out = null;
	SimpleDateFormat formatter
	    = new SimpleDateFormat ("MMM.dd:hh:mm:ss");
	String dateString = formatter.format(now);
	String filename = "log-" + dateString;
	try { out = new PrintWriter(
		    new BufferedWriter(
		    new FileWriter(filename)));
	}
	catch (Exception e) {
	    out = new PrintWriter(new FileWriter(FileDescriptor.out));
	}
	return out;
    }
}             
